#coding:utf-8
import unittest


import ztq_core

def echo():
    print 'hello'

class TestftsModel(unittest.TestCase):
    def setUp(self):
        ztq_core.setup_redis('default', '192.168.209.128', 6379)
        self.testmessage = {'test':'test'}
        
    def testJsonList(self):    
        """Test queue connect
        """
        self.queue = ztq_core.get_task_queue('q01')
        self.queue.append(self.testmessage)
        revmessage = self.queue.pop()
        self.assertEqual(revmessage,self.testmessage)
    
    def _testRegister(self):
        """测试JobThread
        """
        ztq_core.task.register(echo)
        ztq_core.task.task_push(u'foo:echo', 'aaa', 'bb', c='bar') 
        job_thread = ztq_core.task.JobThread('foo')
        job_thread.start()
        
if __name__ == '__main__':
    unittest.main()



#coding:utf-8
from redis_wrap import (
        get_redis, 
        get_list, 
        get_hash, 
        get_set, 
        setup_redis, 
        get_key, 
        set_key, 
        get_queue, 
        get_dict, 
        ConnectionError
        )

from task import (
        task_registry, 
        register, 
        push_task, 
        has_task, 
        pop_task,
        pop_error,
        push_runtime_error,
        gen_task, 
        push_runtime_task
        )

from model import *

from cron import add_cron, has_cron, remove_cron

from async import async, enable_transaction, ping_task, prepare_task




#coding:utf-8
from redis_wrap import get_set, get_key, set_key, \
get_queue, get_dict, get_keys, get_limit_queue, get_hash

def get_all_task_queue():
    """返回所有原子队列的key
            返回类型:list
    """
    task_queue = "ztq:queue:task:"
    return get_keys(task_queue)

def get_all_error_queue():
    """返回所有原子队列的key
            返回类型:list
    """
    task_queue = "ztq:queue:error:"
    return get_keys(task_queue)

def get_task_hash(queue_name):
    """ 得到 一个 task_md5 -> task 的字典对象 """
    return get_hash('ztq:hash:task:' + queue_name)

def get_task_set(queue_name, serialized_type='json'):
    """ 得到 一个 task_md5 -> task 的字典对象 """
    return get_set('ztq:set:task:' + queue_name, serialized_type=serialized_type)

def get_task_queue(queue_name):
    """根据传入参数queue_name

    {"func":'transform',
     'args':(),
     'kw':{'path':'c:\\abc.doc',              #源文件路径
      'mime':'application/ms-word',      #源文件类型
      'transform':[
         {'path':'d:\\abc.html',  
          'mime':'text/html',
          'callback':'http://xxx.com/asss'

          'transform':[
             {'path':'d:\\abc.txt',
              'mime':'text/plain',
              'transform':[]
             }, # 转换子文件放在同一文件夹中
          ]}]},

     'callback':callback,            # 全部完成调用方法
     'callback_args':callback_args,  # 全部完成的调用参数
     'callback_kw':callback_kw,

     'pcallback':callback, # progress callback 部分完成的调用
     'pcallback_args':callback_args, # 部分完成的调用参数
     'pcallback_kw':callback_kw,     # 部分完成调用参数

     'fcallback':callback,            # 失败调用方法
     'fcallback_args':callback_args,  # 失败的调用参数
     'fcallback_kw':callback_kw,

     "runtime":{ # 队列运行相关信息
       'created':12323423   # 进入队列时间
     }
    }
    """
    #ListFu
    atom_queue = "ztq:queue:task:" + queue_name
    return get_queue(atom_queue, serialized_type='string')

def get_command_queue(name):
    """ 同步配置、状态报告、杀死转换线程
    
       要求同步worker配置
    {
     'command':'updateworker',
     'timestamp':''
                   }
        要求同步转换器线程驱动
    {
     'command':'updatedriver',
     'timestamp':''
                   }
                   
        要求worker报告整体工作状态::

     {
     'command':'report',
     'timestamp': 
     }
        后台杀一个转换进程（可能卡死）::
     {
     'command':'kill',
     'timestamp':
     'pid':'2121',
     }
    
    用户取消一个转换进程，和杀死类似，不应该进入错误队列，日志也需要说明是取消::
     {
     'command':'cancel',
     'timestamp':
     'pid':'2121',
     }
    """
    command_queue = 'ztq:queue:command:'+name
    return get_queue(command_queue)

def get_work_log_queue():
    """ json格式为::

     {'func':'transform',
      'kw':{ ... # 和前面task_queue相同
         },
      "runtime":{ # 队列运行相关信息
         'created':12323423   #进入原始队列时间
         'queue':'q01'  # 是在哪个原子原子队列
         'start':123213123    #转换开始时间
         'end':123213123    #转换结束时间
         'worker':'w01', # 转换器名
         'thread':'131231', # 
         'return':-1, # 返回的错误代号, 0表示成功 
         'reason':'失败原因' # 详细的原因
       }
      }
    """
    work__log_queue = "ztq:queue:worker_log"
    return get_limit_queue(work__log_queue, 200)

def get_error_hash(queue_name, system='default'):
    """ json格式和work_log相同 """
    error_queue = 'ztq:hash:error:' + queue_name
    return get_hash(error_queue, system=system)

def get_error_queue(queue_name, system='default'):
    """ json格式和work_log相同 """
    error_queue = 'ztq:queue:error:' + queue_name
    return get_queue(error_queue, system=system, serialized_type='string')

def get_buffer_queue(queue_name, system='default'):
    """ json格式和work_log相同 """
    buffer_queue = 'ztq:queue:buffer:' + queue_name
    return get_queue(buffer_queue, system=system)

def get_system_log_queue():
    """
    Json格式为:
    {'alias':'w01'
     'host':'192.168.1.100'
     'timestamp':123213123
     'type': 'reboot' or 'shutdown' or 'power' 三个值中其中一个
    }
    """
    system__log_queue ='ztq:queue:system_log'
    return get_limit_queue(system__log_queue, 200)

def get_callback_queue():
    callback_queue='ztq:queue:callback'
    return get_queue(callback_queue)

# state -------------------------------------------------------------------
def get_all_worker():
    """返回正在运行中的转换器列表
            返回类似:list
    """
    prefix = 'ztq:state:worker:'
    return get_keys(prefix)

def get_worker_state():
    """ transformer在如下2种状况下会，会由指令线程上报转换器的状态::

    - 启动的时候
    - 有指令要求

        在redis中的存放格式为::

      {'ip':'192.168.1.1',
       'cpu_style':'Dural Xommm 1G',
       'cpu_percent':'30%',
       'mem_total':'2G',
       'mem_percent':'60%',
       'started':1231231231,
       'timestamp':12312312,
       'tracebacks':'全部线程的traceback信息，用于死锁检查',
      }
        转换器状态信息，主要用于监控转换器是否良性工作，会在监控界面中显示。
    """
    prefix = 'ztq:state:worker:'
    return get_dict(prefix)

def get_job_state(worker_job_name):
    """ 转换器w01，第0号转换线程的当前转换任务信息

    - 每次开始转换，需要记录转换的信息
    - 每次结束的时候，需要清空

    json格式为::
    
     {'func':'transform',
      'kw':{ ... # 和上面task_queue相同
         },
      'runtime':{... # 和上面work_log相同
       }
      'process':{
          'pid': 212,  # -1 表示不能杀
          'start':131231,
          'comment':'d:\ssd.pdf'
         }
      }
    """
    prefix = 'ztq:state:job:%s:' % worker_job_name
    return get_dict(prefix)

# config -------------------------------------------------------------------
def get_dispatcher_config():
    """ 用户设定的一组转换参数，在redis中的存放如下信息::
     {'queue_weight':{ # 原子转换队列权重
         'q01':3, 
         'q02':3}
      'worker_weight':{ # 转换器权重
        'w01':3, 
        'w02':0, # 权重为0就禁用
        'w03':4}
     }
    """
    prefix = 'ztq:config:dispatcher'
    return get_key(prefix)

def set_dispatcher_config(value):
    prefix = 'ztq:config:dispatcher'
    return set_key(prefix, value)

def get_queue_config():
    """记录queue的基本信息
    {'title':'sys_cron-0',  #可选
    'tags':(['sys_cron']),  #可选
    'weight':5}             #可选
    """
    prefix = 'ztq:config:queue:'
    return get_dict(prefix)

def get_worker_config():
    """ 配置工作线程：处理哪些队列，几个线程，间隔时间::
    {'q01':[{ 'interval':5,  # 间隔时间
              'from_right':True, }],  # 队列处理的方向，左（l）或者右（r） 
     'q02':[{'interval':5, 'from_right':False},
            {'interval':3, 'from_right':True}],
    }
    """
    prefix = 'ztq:config:worker:'
    return get_dict(prefix)

def get_driver_config():
    """
    TODO:消息格式
    """
    prefix = 'ztq:config:driver:'
    return get_dict(prefix)

def get_cron_set():
    """ 定时任务list
    TODO
    """
    return get_set('ztq:set:cron')
  



#coding:utf-8
from async import async
import redis_wrap
import urllib2
from cron import has_cron, add_cron

async(queue='clock')
def bgrewriteaof():
    """ 将redis的AOF文件压缩 """
    redis = redis_wrap.get_redis()
    redis.bgrewriteaof()

def set_bgrewriteaof():
    # 自动定时压缩reids
    if not has_cron(bgrewriteaof):
        add_cron({'hour':1}, bgrewriteaof)

@async(queue='urlopen')
def async_urlopen(url, params=None):
    try:
        urllib2.urlopen(url, params) # use POST method
    except IOError:
        raise IOError('Could not connected to %s' % url)





'''
Created on 2011-4-18

@author: Zay
'''
from ztq_core import get_redis, get_list, get_hash, get_set, get_dict, setup_redis, \
get_key, set_key, get_queue

def main():
        setup_redis('default', '192.168.209.128', 6380)
        get_redis(system='default').delete('list')
        message = 'hello'
        
        Test_list = get_list('list',serialized_type='string')
        Test_list.append(message)
        
        #Test_list.remove(message)
        
        print get_redis(system='default').lrem('list', 0, 'hello')
        
        Test_set = get_set('set',serialized_type='string')
        Test_set.add(message)
        print get_redis(system='default').srem('set', 'hello')
        
        
if __name__ == '__main__':
    main()



#coding:utf-8

import redis
import pickle
try:
    import json
except :
    import simplejson as json

import UserDict, UserList

ConnectionError = redis.exceptions.ConnectionError

DEFAULT_ENCODING = 'UTF-8' # sys.getdefaultencoding()
#--- System related ----------------------------------------------
SYSTEMS = {
    'default': redis.Redis(host='localhost', port=6379)
}


def setup_redis(name, host, port, db=0, **kw):
    SYSTEMS[name] = redis.Redis(host=host, port=port, db=db, **kw)

def get_redis(system='default'):
    return SYSTEMS[system]

#--- Decorators ----------------------------------------------
def get_list(name, system='default',serialized_type='json'):
    return ListFu(name, system, serialized_type=serialized_type)

def get_queue(name, system='default',serialized_type='json'):
    return QueueFu(name, system, serialized_type=serialized_type)

def get_limit_queue(name, length, system='default',serialized_type='json'):
    return LimitQueueFu(name, length, system, serialized_type=serialized_type)

def get_hash(name, system='default',serialized_type='json'):
    return HashFu(name, system, serialized_type=serialized_type)

def get_set(name, system='default',serialized_type='json'):
    return SetFu(name, system, serialized_type=serialized_type)

def get_dict(name, system='default',serialized_type='json'):
    return DictFu(name, system, serialized_type=serialized_type)

def get_key(name, system='default',serialized_type='json'):
    loads = load_method[serialized_type]
    value = get_redis(system).get(name)
    try:
        return loads(value)
    except:return value

def del_key(name, system='default'):
    get_redis(system).delete(name)
    
def get_keys(name, system='default'):
    for key in get_redis(system).keys(name + "*"):
        key_name = key[len(name):]
        yield key_name
   
def set_key(name, value, system='default',serialized_type='json'):
    dumps = dump_method[serialized_type]
    value = dumps(value)
    get_redis(system).set(name, value)

#---serialize data type----------------------------------------
def _convert_persistent_obj(obj):
    # fix json.dumps raise TypeError
    # 是persistent 对象
    if isinstance(obj, (UserDict.UserDict, dict)):
        return dict(obj)
    elif isinstance(obj, (UserList.UserList, list, set)):
        return list(obj)
    raise TypeError, '%s: %s is not JSON serializable'%(type(obj), repr(obj))

dump_method = {'json':lambda item : json.dumps(item, sort_keys=True, \
                encoding=DEFAULT_ENCODING, default=_convert_persistent_obj),
               'pickle':pickle.dumps,
               'string':str
               }
load_method = {'json':json.loads,
               'pickle':pickle.loads,
               'string':str
               }
    
#--- Data impl. ----------------------------------------------
class ListFu(object):

    def __init__(self, name, system, serialized_type='json'):
        self.name = name
        self.system = system
        self.type = serialized_type
        self.dumps = dump_method[serialized_type]
        self.loads = load_method[serialized_type]
        
    def append(self, item):
        item = self.dumps(item)
        get_redis(self.system).lpush(self.name, item)

    def extend(self, iterable):
        for item in iterable:
            self.append(item)
    
    def remove(self, value):
        value = self.dumps(value)
        get_redis(self.system).lrem(self.name, value)

    def pop(self, index=None):
        if index:
            raise ValueError('Not supported')
        serialized_data = get_redis(self.system).rpop(self.name)
        if serialized_data[1]:
            item = self.loads(serialized_data[1])
            return item
        else: return None

    def __len__(self):
        return get_redis(self.system).llen(self.name)

    def __iter__(self):
        client = get_redis(self.system)
        i = 0
        while True:
            items = client.lrange(self.name, i, i+30)
            if len(items) == 0:
                break
                #raise StopIteration
            for item in items:
                yield self.loads(item)
            i += 30

    def __getitem__(self, index):
        client = get_redis(self.system)
        value = client.lindex(self.name, index)
        return self.loads(value) if value else None

    def __getslice__(self, i, j):
        client = get_redis(self.system)
        items = client.lrange(self.name, i, j)
        for item in items:
            yield self.loads(item)

class HashFu:

    def __init__(self, name, system, serialized_type='json'):
        self.name = name
        self.system = system
        self.dumps = dump_method[serialized_type]
        self.loads = load_method[serialized_type]

    def get(self, key, default=None):
        value = get_redis(self.system).hget(self.name, key)
        try:
            return self.loads(value)
        except: return default
        
    def items(self):
        for key in self.keys():
            # key_list 不是实时的数据
            # 这个任务可能已经被取走了（当监视这个队列的工作线程有多个的时候）
            value = self.get(key)
            if value is None: continue

            yield key, value

    def keys(self):
        return get_redis(self.system).hkeys(self.name) or []

    def values(self):
        _values = self.loads(get_redis(self.system).hvals(self.name))
        return _values or []

    def pop(self, key):
        pline = get_redis(self.system).pipeline()
        pline.hget(self.name, key).hdel(self.name, key)
        _value, _expire = pline.execute()
        if _expire:
            return self.loads(_value)
        else:
            #raise KeyError,'redis hasher not match the %s key\n\n'%key
            print 'redis hasher not match the %s key\n\n'%key
            return None

    def __len__(self):
        return get_redis(self.system).hlen(self.name) or 0

    def __getitem__(self, key):
        val = self.get(key)
        if not val:
            raise KeyError
        return val

    def __setitem__(self, key, value):
        value = self.dumps(value)
        return get_redis(self.system).hset(self.name, key, value)

    def __delitem__(self, key):
        get_redis(self.system).hdel(self.name, key)

    def __contains__(self, key):
        return get_redis(self.system).hexists(self.name, key)

    def update(self, new_dict, **kw):
        update = {}

        if new_dict and hasattr(new_dict, 'keys'):
            for key in new_dict:
                update[key] = self.dumps(new_dict[key])
        elif new_dict:
            for key, value in new_dict:
                update[key] = self.dumps(key)

        for key in kw:
            update[key] = self.dumps(key[key])

        if update:
            get_redis(self.system).hmset(self.name, update)

class SetFu:

    def __init__(self, name, system, serialized_type='json'):
        self.name = name
        self.system = system
        self.dumps = dump_method[serialized_type]
        self.loads = load_method[serialized_type]
        
    def add(self, item):
        item = self.dumps(item)
        get_redis(self.system).sadd(self.name, item)

    def remove(self, item):
        item = self.dumps(item)
        get_redis(self.system).srem(self.name, item)

    def pop(self, item):
        item = self.serializer.dumps(item)
        value = get_redis(self.system).spop(self.name, item)
        return self.loads(value)

    def __iter__(self):
        client = get_redis(self.system)
        for item in client.smembers(self.name):
            yield self.loads(item)

    def __len__(self):
        return len(get_redis(self.system).smembers(self.name))

    def __contains__(self, item):
        item = self.dumps(item)
        return get_redis(self.system).sismember(self.name, item)

class DictFu:
    
    def __init__(self, name, system, serialized_type='json'):
        self.name = name
        self.system = system
        self.dumps = dump_method[serialized_type]
        self.loads = load_method[serialized_type]
    
    def get(self, key, default=None):
        value = get_redis(self.system).get(self.name+key)
        try:
            return self.loads(value)
        except: return default
        
    def set(self, key, value):
        value = self.dumps(value)
        get_redis(self.system).set(self.name+key, value)
    
    def __delitem__(self, key):
        get_redis(self.system).delete(self.name+key)
    
    def __len__(self):
        listkey = get_redis(self.system).keys(self.name+"*")
        return len(listkey) or 0

    def keys(self):
        prefix_len = len(self.name)
        return [key[prefix_len:] for key in get_redis(self.system).keys(self.name + "*")]

    def items(self):
        # XXX self.get 每次都要连结redis， 这样不好
        key_list = get_redis(self.system).keys(self.name+"*")
        for key in key_list:
            key_name = key[len(self.name):]

            # key_list 不是实时的数据
            # 这个任务可能已经被取走了（当监视这个队列的工作线程有多个的时候）
            value = self.get(key_name)
            if value is None: continue

            yield key_name, value
    
    def __getitem__(self, key=''):
        val = self.get(key, None)
        if val is None:
            raise KeyError
        return val
    
    def __setitem__(self, key, value):
        self.set(key, value)
    
    def __contains__(self, key):
        return get_redis(self.system).exists(self.name+key)

class QueueFu(ListFu):

    def __init__(self, name, system, serialized_type='json'):
        super(QueueFu,self).__init__(name, system, serialized_type=serialized_type)
 
    def push(self, item, to_left=True):
        if to_left:
            self.append(item)
        else: 
            item = self.dumps(item)
            get_redis(self.system).rpush(self.name, item)
            
    def pop(self, timeout=0, from_right = True):
        """ 
            得到redis list 对象中的一个item，并把item 从 redis list 对象中删除
            from_right: 如果值为真，从redis list 对象右边读取，反之，从左边读取
            timeout: timeout 等于大于0，以阻塞式获取。timeout 小于0，直接获取返回
        """
        if from_right:
            if timeout >= 0:
                serialized_data = get_redis(self.system).brpop(self.name, timeout)
            else:
                serialized_data = get_redis(self.system).rpop(self.name)
        else:
            if timeout >= 0:
                serialized_data = get_redis(self.system).blpop(self.name, timeout)
            else:
                serialized_data = get_redis(self.system).lpop(self.name)

        if serialized_data:
            # 阻塞式获取，返回self.name, result
            if isinstance(serialized_data, (tuple, list, set)) and \
                    len(serialized_data) == 2:
                return self.loads(serialized_data[1]) if serialized_data[1] else None
            # 直接获取，返回 result
            else:
                return self.loads(serialized_data)

        return None
     
    def reverse(self):
        """倒序输出结果 
        """
        client = get_redis(self.system)
        length = client.llen(self.name)
        for index in xrange(length-1, -1, -1):
            item = client.lindex(self.name, index)
            yield self.loads(item)
        
class LimitQueueFu(QueueFu):
    """此队列类用于控制队列长度，主要用于日志
    """
    def __init__(self, name, length, system, serialized_type='json'):
        super(LimitQueueFu,self).__init__(name, system, serialized_type=serialized_type)
        self.length = length - 1
        
    def push(self, item):
        #QueueFu.push(self, item)
        #get_redis(self.system).ltrim(self.name, 0, self.length)

        item = self.dumps(item)
        pline = get_redis(self.system).pipeline()
        pline.lpush(self.name, item).ltrim(self.name, 0, self.length)
        pline.execute()




#coding:utf-8
'''
测试说明:
此测试是针对redis_wrap库进行自动json编码的测试

测试结果:
Ran 5 tests in 0.036s

FAILED (failures=1)

失败原因是在对list进行remove(value)操作的时候,redis的lrem无法删除序列化后的对象,
set类型能正常remove序列化后的对象.

@author: Zay
'''
import unittest
from ztq_core import get_redis, get_list, get_hash, get_set, get_dict, setup_redis, \
get_key, set_key, get_queue

class TestRediswrap(unittest.TestCase):
    def setUp(self):
        """初始化连接redis,和初始化变量
        """
        setup_redis('default', '192.168.209.128', 6379, socket_timeout=2)
        get_redis(system='default').delete('list')
        get_redis(system='default').delete('set')
        get_redis(system='default').delete('hash')
        get_redis(system='default').delete('dict')
        get_redis(system='default').delete('kv')
        get_redis(system='default').delete('queue')
        self.message = {"hello":"grizzly"}
        
    def test_getset(self):
        """进行基本的redis 的key进行get和set的操作.
        """
        Test_key = get_key('kv',serialized_type='json')
        self.assertEqual(Test_key,None)
        
        set_key('kv',self.message)
        
        Test_key = get_key('kv',serialized_type='json')
        self.assertEqual(Test_key,self.message)
        
    def test_dict(self):
        """测试redis_wrap的dict类型的操作
        """
        Test_dict = get_dict('dict',serialized_type='json')
        
        Test_dict['id'] = self.message
        self.assertEqual(self.message, Test_dict['id'])
        
        for k,v in Test_dict.items():
            self.assertEqual(k, 'id')
            self.assertEqual(v, self.message)
        
        del Test_dict['id']
        self.assertNotEqual(self.message,Test_dict.get('id'))
        
    def test_hash(self):
        """测试redis_wrap的 hash类型的操作
        """
        Test_dict = get_hash('hash',serialized_type='json')
        
        Test_dict['id'] = self.message
        self.assertEqual(self.message, Test_dict['id'])
        
        del Test_dict['id']
        self.assertNotEqual(self.message,Test_dict.get('id'))        
        
    def test_list(self):
        """进行redis_wrap的list的基本操作
        """
        Test_list = get_list('list',serialized_type='json')
        
        Test_list.append(self.message)
        self.assertEqual( len(Test_list),1)
    
        for item in Test_list:
            self.assertEqual(self.message, item)
            
        #这一步失败原因是redis的lrem方法有无法删除序列化后的数据
        Test_list.remove(self.message)
        self.assertEqual( len(Test_list),0)

    def test_set(self):
        """进行对redis_wrap的set类型的基本操作
        """
        Test_set = get_set('set',serialized_type='json')
        Test_set.add(self.message)
        
        for item in Test_set:
            self.assertEqual( item,self.message)
            
        Test_set.remove(self.message)
        self.assertEqual( len(Test_set),0)

    def test_queue(self):
        """进行redis_wrap的queue的基本操作
        """
        Test_queue = get_queue('queue',serialized_type='json')
        
        Test_queue.push(self.message)
        self.assertEqual( len(Test_queue),1)
    
        for item in Test_queue:
            self.assertEqual(self.message, item)
            
        #这一步失败原因是redis的lrem方法有无法删除数据
        Test_queue.remove(self.message)
        self.assertEqual( len(Test_queue),0)
    #===========================================================================
    # 
    #    message = Test_queue.pop(timeout= 1)
    #    self.assertEqual(self.message, message)
    #    self.assertEqual(len(Test_queue),0)
    #===========================================================================
        
if __name__ == '__main__':
    unittest.main()




import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.txt')).read()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()

requires = [ 'redis>=2.4.9', 'transaction',]

if not os.sys.platform.startswith('win'):
    requires.append('hiredis')

setup(name='ztq_core',
      version = '1.2.1',
      author="edo",
      author_email="service@everydo.com",
      url="http://everydo.com/",
      description=u"Zopen Task Queue Core",
      long_description=README + '\n\n' +  CHANGES,
      packages=find_packages(),
      license = "MIT",
      platforms=["Any"],
      keywords='Everydo queue ztq_core async',
      classifiers = [
          'Development Status :: 4 - Beta',
          'Environment :: Web Environment',
          'Intended Audience :: Developers',
          'Programming Language :: Python',
          'Operating System :: OS Independent',
          'Topic :: Internet :: WWW/HTTP',
          ],
	  install_requires = requires,
)




# encoding: utf-8
#  my_send.py
from ztq_core import async
import ztq_worker
import time

@async(queue='mail')
def send(body):
    print 'START: ', body
    time.sleep(5)
    print 'END: ', body

@async(queue='mail')
def send2(body):
    print 'START2 … ', body
    raise Exception('connection error')

@async(queue='mail')
def call_process(filename):
    print 'call process:', filename
    ztq_worker.report_job(12323, comment=filename)
    time.sleep(20)

@async(queue='mail')
def fail_callback(return_code, return_msg):
    print 'failed, noe in failed callback'
    print return_code, return_msg

def test():
    import ztq_core
    import transaction

    from ztq_core import demo

    ztq_core.setup_redis('default','localhost', 6379, 1)

    demo.send('*' * 40, ztq_queue='mail')

    demo.send('transaction will on', ztq_queue='mail')
    ztq_core.enable_transaction(True)

    demo.send('transaction msg show later')
    demo.send('no transaction msg show first', ztq_transaction=False)
    time.sleep(5)
    transaction.commit()
    
    ztq_core.enable_transaction(False)

    demo.send('transaction off')
    callback = ztq_core.prepare_task(demo.send, 'yes, callback!!')
    demo.send('see callback?', ztq_callback=callback)

    ff = ztq_core.prepare_task(demo.fail_callback)
    demo.send2('send a failed msg, see failed callback?', ztq_fcallback=ff)

    call_process('saa.exe')

if __name__ == '__main__':
    test()



#coding:utf-8

""" 有一个定时执行的list: ztq:list:cron

放进去的工作，会定期自动执行
"""
from threading import Thread
import datetime
import time
import model
from task import split_full_func_name, push_task


def has_cron(func):
    if type(func) == str:
        func_name = func
    else:
        func_name = func.__raw__.__name__
    for cron in model.get_cron_set():
        if cron['func_name'] == func_name:
            return True
    return False

def add_cron(cron_info, full_func, *args, **kw):
    """ 定时执行 

    cron_info： {'minute':3, 'hour':3,}
    """
    cron_set = model.get_cron_set()
    if type(full_func) == str:
        queue_name, func_name = split_full_func_name(full_func)
    else:
        queue_name = full_func._ztq_queue
        func_name = full_func.__raw__.__name__
    cron_set.add({'func_name':func_name, 
                'cron_info':cron_info,
                'queue': queue_name,
                'args':args,
                'kw':kw})

def remove_cron(func):
    cron_set = model.get_cron_set()
    if type(func) == str:
        func_name = func
    else:
        func_name = func.__raw__.__name__
    for cron in cron_set:
        if cron['func_name'] == func_name:
            cron_set.remove(cron)

class CronThread(Thread):
    """ 定时检查cron列表，如果满足时间条件，放入相关的队列 """
    def __init__(self):
        super(CronThread, self).__init__()

    def run(self):
        """
            获取cron_info信息格式:{'minute':3, 'hour':3,}
        """
        cron_set = model.get_cron_set()
        while True:
            # 遍历cron列表检查并检查定时执行信息
            for cron in cron_set:
                execute_flag = self.check_cron_info(cron['cron_info'])
                if execute_flag:
                    push_task(cron['queue'] + ':' + cron['func_name'], *cron['args'], **cron['kw'])

            time.sleep(55)

    def check_cron_info(self, cron_info):
        """检查定时执行信息是否满足条件
        """
        time_now = datetime.datetime.now()
        hour_cron = int(cron_info.get('hour', -1)) 
        if hour_cron != -1:
            hour_now = int(time_now.hour)
            if hour_now != hour_cron:
                return False

        minute_cron = int(cron_info.get('minute', 0))
        minute_now = int(time_now.minute)
        if minute_cron != minute_now:
            return False
        return True





#coding:utf-8
""" redis任务队列
"""

import model
import time
from threading import Thread
from hashlib import md5
from redis_wrap import dump_method

task_registry = {}

def register(func, func_name = None):
    """ 注册task

    定义::

      def echo(aaa, bb, c=1):
          print aaa, bb, c

    注册远端任务::

      from zopen_redis import task_registry
      task_registry.register(echo)
    """
    task_registry[func_name or func.__name__] = func

def split_full_func_name(full_func_name):
    splitted_func_name = full_func_name.rsplit(':', 1)
    # 如果没有，就到默认队列
    if len(splitted_func_name) == 1:
        return 'default', full_func_name
    else:
        return splitted_func_name

def gen_task(func_name, *args, **kw):
    callback = kw.pop('ztq_callback', '')
    callback_args = kw.pop('ztq_callback_args', ()) 
    callback_kw = kw.pop('ztq_callback_kw', {})

    fcallback = kw.pop('ztq_fcallback', '')
    fcallback_args = kw.pop('ztq_fcallback_args', ()) 
    fcallback_kw = kw.pop('ztq_fcallback_kw', {})

    pcallback = kw.pop('ztq_pcallback', '')
    pcallback_args = kw.pop('ztq_pcallback_args', ()) 
    pcallback_kw = kw.pop('ztq_pcallback_kw', {})
    return {'func':func_name,
                'args':args,
                'kw':kw, 

                'callback':callback,
                'callback_args':callback_args,
                'callback_kw':callback_kw,

                'fcallback':fcallback,
                'fcallback_args':fcallback_args,
                'fcallback_kw':fcallback_kw,

                'pcallback':pcallback,
                'pcallback_args':pcallback_args,
                'pcallback_kw':pcallback_kw,
            }

def _get_task_md5(task):
    """ 得到task(dict) 的md5值 """
    #_value = json.dumps(task, sort_keys=True)
    _value = dump_method['json'](task)

    return md5(_value).digest()

def push_buffer_task(full_func_name, *args, **kw):
    queue_name, func_name = split_full_func_name(full_func_name)
    task = gen_task(func_name, *args, **kw)
    model.get_buffer_queue(queue_name).push(task)

def push_task(full_func_name, *args, **kw):
    """
    callback: 这是另外一个注册的task，在func调用完毕后，会启动这个

    加入队列::

     task_regitry.push(u'foo:echo', aaa, bb, foo='bar', 
            callback='foo:callback', callback_args=(12,32,3), callback_kw={}) 
    """
    queue_name, func_name = split_full_func_name(full_func_name)
    to_right = kw.pop('ztq_first', False)
    # 队列运行相关信息
    runtime = kw.pop('runtime', \
            {'create':int(time.time()), 'queue':queue_name})

    task = gen_task(func_name, *args, **kw)
    task_md5 = _get_task_md5(task)

    task_hash = model.get_task_hash(queue_name)

    # 因为queue队列有worker不停在监视,必须先将hash的内容push,在将queue的内容push
    task['runtime'] = runtime
    if task_hash.__setitem__(task_md5, task) == 1:
        # 如果返回值等于0， 说明task_md5已经存在
        queue = model.get_task_queue(queue_name)
        queue.push(task_md5, to_left=not to_right)

def push_runtime_task(queue_name, task):
    """ 直接将task push 到 redis """
    _push_runtime_job(queue_name, task, model.get_task_hash, model.get_task_queue)

def push_runtime_error(queue_name, error):
    _push_runtime_job(queue_name, error, model.get_error_hash, model.get_error_queue)

def _push_runtime_job(queue_name, task, get_hash, get_queue):
    to_left = task.get('kw', {}).pop('to_left', True)
    runtime = task.pop('runtime') 

    task_md5 = _get_task_md5(task)
    task_hash = get_hash(queue_name)

    # 因为queue队列有worker不停在监视,必须先将hash的内容push,在将queue的内容push
    task['runtime'] = runtime
    if task_hash.__setitem__(task_md5, task) == 1:
        # 如果返回值等于0， 说明task_md5已经存在
        queue = get_queue(queue_name)
        queue.push(task_md5, to_left=to_left)

def pop_task(queue_name, task_md5=None, timeout=0, from_right=True):
    """ 取出，并删除 """
    return _pop_job(queue_name, task_md5, 
            model.get_task_hash, model.get_task_queue, timeout, from_right)

def pop_error(queue_name, task_md5=None, timeout=0, from_right=True):
    return _pop_job(queue_name, task_md5, 
            model.get_error_hash, model.get_error_queue, timeout, from_right)

def _pop_job(queue_name, task_md5, get_hash, get_queue, timeout=0, from_right=True):

    if not task_md5:
        task_md5 = get_queue(queue_name).pop(timeout=timeout, \
                from_right=from_right)
    else:
        get_queue(queue_name).remove(task_md5)

    if not task_md5: return None # 可能超时了

    task_hash = get_hash(queue_name)
    return task_hash.pop(task_md5)

class JobThread(Thread):
    def __init__(self,queue_name):
        super(JobThread,self).__init__()
        self.queue_name = queue_name

    def run(self):
        """ 阻塞方式找到任务，并自动调用"""
        queue = model.get_task_queue(self.queue_name)
        while True:
            task = queue.pop()
            try:
                task_registry[task['func']](*task['args'], **task['kw'])
                if task['callback']:
                    callback_args = task.get('callback_args', ())
                    callback_kw = task.get('callback_kw', {})
                    push_task(task['callback'], *callback_args, **callback_kw)
            except Exception, e:
                print str(e)

def has_task(queue_name, task, to_front=False):
    """ 检查是否存在某个job
    在queue_name的队列上，在arg_index的位置，对于func_name, 值为arg_value 
    如果不存在，返回false， 在worker中工作，返回‘work'， 队列中返回’queue'
    """
    runtime = task.pop('runtime', None)
    task_md5 = _get_task_md5(task)
    if not runtime is None: task['runtime'] = runtime

    # 检查work现在的工作
    worker_list = model.get_all_worker()
    for worker_name in worker_list:
        worker_job = model.get_job_state(worker_name)
        if not worker_job: continue
        for thread_name, job in worker_job.items():
            job.pop('runtime', '')
            job.pop('process', '')
            if _get_task_md5(job) == task_md5:
                return 'running'

    # 检查所在队列
    queue_name = queue_name 
    task_hash = model.get_task_hash(queue_name)
    if task_md5 in task_hash:
        if to_front: # 调整顺序
            task_queue = model.get_task_queue(queue_name)
            task_queue.remove(task_md5)
            task_queue.push(task_md5, to_left=False)
        return 'queue'

    return 'none'




# -*- encoding:utf-8 -*-

import types
from task import register, push_task, has_task, gen_task, push_buffer_task
import transaction

use_transaction = False

def _setup_callback(kw):
    callback = kw.pop('ztq_callback', None) 
    if callback is not None:
        callback_func, callback_args, callback_kw = callback
        callback_queue = callback_kw.pop('ztq_queue', callback_func._ztq_queue)
        kw.update({'ztq_callback':"%s:%s" % (callback_queue, callback_func.__raw__.__name__),
                   'ztq_callback_args':callback_args,
                   'ztq_callback_kw':callback_kw})
    fcallback = kw.pop('ztq_fcallback', None) 
    if fcallback is not None:
        callback_func, callback_args, callback_kw = fcallback
        callback_queue = callback_kw.pop('ztq_queue', callback_func._ztq_queue)
        kw.update({'ztq_fcallback':"%s:%s" % (callback_queue, callback_func.__raw__.__name__),
                   'ztq_fcallback_args':callback_args,
                   'ztq_fcallback_kw':callback_kw})
    pcallback = kw.pop('ztq_pcallback', None) 
    if pcallback is not None:
        callback_func, callback_args, callback_kw = pcallback
        callback_queue = callback_kw.pop('ztq_queue', callback_func._ztq_queue)
        kw.update({'ztq_pcallback':"%s:%s" % (callback_queue, callback_func.__raw__.__name__),
                   'ztq_pcallback_args':callback_args,
                   'ztq_pcallback_kw':callback_kw})

def push_task_to_queue(task_name, args, kw, on_commit=False, buffer=False):
    if on_commit:
        if buffer:
            add_after_commit_hook(push_buffer_task, (task_name,) + args, kw)
        else:
            add_after_commit_hook(push_task, (task_name,) + args, kw)
    else:
        if buffer:
            push_buffer_task(task_name, *args, **kw)
        else:
            push_task(task_name, *args, **kw)

def async(*_args, **_kw):
    """ 这是一个decorator，事务提交的时候，提交到job队列，异步执行 

    定义job
    =============
    第一种::

        @async
        def say_hello(name):
            print 'hello, ', name

    第二种, 预先指定队列执行信息::

        @async(queue='hello_queue', transaction=True)
        def say_hello(name):
            print 'hello, ', name

    使用方法
    ================
    支持如下几种::

        say_hello('asdfa')
        say_hello('asdfa', ztq_queue="asdfa", ztq_transaction=False)

    """
    if len(_args) == 1 and not _kw and isinstance(_args[0], types.FunctionType): # 不带参数的形式
        func = _args[0]
        def new_func1(*args, **kw):
            queue_name = kw.pop('ztq_queue', 'default')
            buffer = kw.pop('ztq_buffer', False)
            on_commit= kw.pop('ztq_transaction', use_transaction) 
            task_name = "%s:%s" % (queue_name, func.__name__)
            _setup_callback(kw)
            push_task_to_queue(task_name, args, kw, on_commit=on_commit, buffer=buffer)

        new_func1.__raw__ = func
        new_func1._ztq_queue = 'default'
        register(func)
        return new_func1
    else:
        _queue_name = _kw.get('queue', 'default')
        def _async(func):
            def new_func(*args, **kw):
                #on_commit= kw.pop('ztq_transaction', _on_commit) 
                on_commit= kw.pop('ztq_transaction', use_transaction) 
                queue_name = kw.pop('ztq_queue', _queue_name)
                buffer = kw.pop('ztq_buffer', False)
                task_name = "%s:%s" % (queue_name, func.__name__)
                _setup_callback(kw)
                push_task_to_queue(task_name, args, kw, on_commit=on_commit, buffer=buffer)

            new_func.__raw__ = func
            new_func._ztq_queue = _queue_name
            register(func)
            return new_func
        return _async

def prepare_task(func, *args, **kw):
    _setup_callback(kw)
    return func, args, kw

def ping_task(func, *args, **kw):
    queue_name = kw.pop('ztq_queue', func._ztq_queue)
    to_front = kw.pop('ztq_first', False)
    on_commit = kw.pop('ztq_transaction', None)
    run = kw.pop('ztq_run', False)
    task = gen_task(func.__raw__.__name__, *args, **kw)
    result = has_task(queue_name, task, to_front=to_front)
    if result == 'none' and run:
        kw['ztq_queue'] = queue_name
        kw['ztq_first'] = to_front
        if on_commit is not None:
            kw['ztq_transaction'] = on_commit
        func(*args, **kw)
    return result

#### 以下代码让队列的任务支持事务
def enable_transaction(enable):
    """ 是否支持transaction, 默认不支持 """
    global use_transaction
    use_transaction = bool(enable)

def _run_after_commit(success_commit, func, args, kw):
    if success_commit:
        func(*args, **kw)

def add_after_commit_hook(func, args, kw):
    """ 在事务最后添加一个钩子，让队列任务在事务完成后才做实际的操作
    """
    if not use_transaction: return 
    transaction.get().addAfterCommitHook(
                        _run_after_commit,
                        (func, args, kw),
                        )
